/******************************/
/* 簡単な待ち行列問題（M/M/s）*/
/*      coded by Y.Suganuma   */
/******************************/
base = null;
cus = new Array();
que = new Array();

/****************/
/* main program */
/****************/
function main()
{
	cus.splice(0);   // 系内にいる客のリスト
	que.splice(0);   // Queueオブジェクトリスト
/*
          入力データ
*/
	let s   = parseInt(document.getElementById("s").value);   // 窓口の数
	let end = parseFloat(document.getElementById("end").value);   // シミュレーション終了時間
	let m_a = parseFloat(document.getElementById("m_a").value);   // 平均到着時間間隔
	let m_s = parseFloat(document.getElementById("m_s").value);   // 平均サービス時間
/*
          システムの定義
*/
	base = new Q_base(s, m_a, m_s, end);
	base.at = base.p_time + base.Next_at();   // 最初の客の到着時間
/*
          シミュレーションの実行
*/
	base.Control();
/*
          出力
*/
	base.Output();
}

/******************************/
/* Customerオブジェクトの定義 */
/******************************/
function Customer(n, s, t)
{
	this.time  = t;   // 到着時刻
	this.num   = n;   // 客番号
	this.state = s;   // 客の状態
	                  //   =-1 : 待ち行列に入っている
	                  //   >=0 : サービスを受けている窓口番号

	return this;
}

/**********************/
/* Q_baseオブジェクト */
/**********************/
function Q_base(s, m_a, m_s, end) {
/*
          入力データの設定
*/
	this.s = s;   // 窓口の数
	this.m_a = m_a;   // 到着時間間隔の平均値
	this.m_s = m_s;   // サービス時間の平均値
	this.end = end;   // シミュレーション終了時刻
/*
          初期設定
*/
	this.asb = s;   // 全窓口の空き状況
	                //    =0 : すべての窓口がふさがっている
	                //    =n : ｎ個の窓口が空いている
	this.sb = new Array();   // 各窓口の空き状況
	                         //    =0 : 窓口は空いている
	                         //    >0 : サービスを受けている客番号
	this.c_sb = new Array();   // 各窓口がふさがっている時間の累計
	this.st_e = new Array();   // 各窓口のサービス終了時刻
	this.st_s = new Array();   // 各窓口がふさがった時刻
	for (let i1 = 0; i1 < this.s; i1++) {
		this.sb[i1]   = 0;
		this.c_sb[i1] = 0.0;
	}
	this.asb_t;   // すべての窓口がふさがった時刻
	this.c_asb = 0.0;   // すべての窓口がふさがっている時間の累計
	this.m_lq = 0;   // 最大待ち行列長
	this.c_lq_t = 0.0;   // 待ち行列長にその長さが継続した時間を乗じた値の累計
	this.c_wt = 0.0;   // 待ち時間の累計
	this.lq_t = 0.0;   // 現在の待ち行列長になった時刻
	this.m_wt = 0.0;   // 最大待ち時間
	this.c_sc_t = 0.0;   // 系内客数にその数が継続した時間を乗じた値の累計
	this.c_sys = 0.0;   // 滞在時間の累計
	this.m_sys = 0.0;   // 最大滞在時間
	this.sc_t = 0.0;   // 現在の系内客数になった時刻
	this.m_sc = 0;   // 最大系内客数
	this.at;   // 次の客の到着時刻（負の時は，終了）
	this.p_time = 0.0;   // 現在時刻
	this.nc = 0;   // 到着客数カウンタ

	return this;
}

/********************************/
/* 次の客の到着までの時間の発生 */
/********************************/
Q_base.prototype.Next_at = function()
{
	return -base.m_a * Math.log(Math.random());
}

/**********************/
/* サービス時間の発生 */
/**********************/
Q_base.prototype.Next_sv = function()
{
	return -base.m_s * Math.log(Math.random());
}

/**************/
/* 全体の制御 */
/**************/
Q_base.prototype.Control = function()
{
	let sw = 0;
	while (sw > -2) {
		sw = base.Next();   // 次の処理の選択
		if (sw == -1)
			sw = base.End_o_s();   // シミュレーションの終了
		else {
			if (sw == 0)
				base.Arrive();   // 客の到着処理
			else
				base.Service(sw);   // サービスの終了
		}
	}
}

/**************************************************/
/* 次の処理の決定                                 */
/*      return : =-1 : シミュレーションの終了     */
/*               =0  : 客の到着処理               */
/*               =i  : ｉ番目の窓口のサービス終了 */
/**************************************************/
Q_base.prototype.Next = function()
{
	let sw = -1;
	let t  = base.end;   // シミュレーション終了時刻で初期設定
					// 次の客の到着時刻
	if (base.at >= 0.0 && base.at < t) {
		sw = 0;
		t  = base.at;
	}
					// サービス終了時刻
	for (let i1 = 0; i1 < base.s; i1++) {
		if (base.sb[i1] > 0 && base.st_e[i1] <= t) {
			sw = i1 + 1;
			t  = base.st_e[i1];   // 窓口i1のサービス終了時刻
		}
	}

	return sw;
}

/**********************************/
/* 終了処理                       */
/*      return : =-1 : 終了前処理 */
/*               =-2 : 実際の終了 */
/**********************************/
Q_base.prototype.End_o_s = function()
{
	let sw      = -2;
	base.p_time = base.end;   // 現在時刻
	base.at     = -1.0;   // 次の客の到着時刻

	for (let i1 = 0; i1 < base.s; i1++) {
		if (base.sb[i1] > 0) {   // サービス中の客はいるか？
			if (sw == -2) {
				sw = -1;
				base.end = base.st_e[i1];   // 窓口i1のサービス終了時刻
			}
			else {
				if (base.st_e[i1] > base.end)
					base.end = base.st_e[i1];   // 窓口i1のサービス終了時刻
			}
		}
	}

	return sw;
}

/****************/
/* 客の到着処理 */
/****************/
Q_base.prototype.Arrive = function()
{
/*
          客数の増加と次の客の到着時刻の設定
*/
	base.nc     += 1;   // 到着客数カウンタ
	base.p_time  = base.at;   // 現在時刻
	base.at      = base.p_time + base.Next_at();   // 次の客の到着時刻
	if (base.at >= base.end)
		base.at = -1.0;
/*
          系内客数の処理
*/
	base.c_sc_t += cus.length * (base.p_time - base.sc_t);   // 系内客数にその数が継続した時間を乗じた値の累計
	base.sc_t    = base.p_time;   // 現在の系内客数になった時刻
	if (cus.length+1 > base.m_sc)
		base.m_sc = cus.length + 1;   // 最大系内客数
/*
          窓口に空きがない場合
*/
	if (base.asb == 0) {
		let ct = new Customer(base.nc, -1, base.p_time);
		cus.push(ct);   // 客の登録（系内客数）
		base.c_lq_t += que.length * (base.p_time - base.lq_t);   // 待ち行列長にその長さが継続した時間を乗じた値の累計
		que.push(base.nc);   // 客の登録（待ち行列）
		base.lq_t = base.p_time;   // 現在の待ち行列長になった時刻
		if (que.length > base.m_lq)
			base.m_lq = que.length;   // 最大待ち行列長
	}
/*
          すぐサービスを受けられる場合
*/
	else {
		let k = -1;
		for (let i1 = 0; i1 < base.s && k < 0; i1++) {
			if (base.sb[i1] == 0) {
				let ct = new Customer(base.nc, i1, base.p_time);
				cus.push(ct);   // 客の登録（系内客数）
				k = i1;
				base.sb[k]    = base.nc;   // サービスを受けている客番号
				base.st_e[k]  = base.p_time + base.Next_sv();   // 窓口kのサービス終了時刻
				base.asb     -= 1;   // 空いている窓口数
			}
		}
		base.st_s[k] = base.p_time;   // 窓口kがふさがった時刻
		if (base.asb == 0)
			base.asb_t = base.p_time;   // すべての窓口がふさがった時刻
	}
}

/*********************************/
/* サービス終了時の処理          */
/*      k : サービス終了窓口番号 */
/*********************************/
Q_base.prototype.Service = function(k)
{
/*
          時間の設定
*/
	k -= 1;
	base.p_time  = base.st_e[k];   // 現在時刻
	base.st_e[k] = -1.0;   // サービス終了時間
/*
          系内客数の処理
*/
	base.c_sc_t += cus.length * (base.p_time - base.sc_t);   // 系内客数にその数が継続した時間を乗じた値の累計
	base.sc_t    = base.p_time;   // 現在の系内客数になった時刻
/*
          滞在時間の処理
*/
	let n  = base.Search(base.sb[k]);
	let x1 = base.p_time - cus[n].time;
	base.c_sys += x1;   // 滞在時間の累計
	if (x1 > base.m_sys)
		base.m_sys = x1;   // 最大滞在時間
	cus.splice(n, 1);   // 客の削除（系内客数）
/*
          窓口使用時間の処理
*/
	base.asb     += 1;   // 空いている窓口数
	base.sb[k]    = 0;   // 窓口kを空き状態にする
	base.c_sb[k] += (base.p_time - base.st_s[k]);   // 窓口kがふさがっている時間の累計
/*
          待ち行列がある場合
*/
	if (que.length > 0) {
		base.asb    -= 1;   // 開いている窓口数
		base.c_lq_t += que.length * (base.p_time - base.lq_t);   // 待ち行列長にその長さが継続した時間を乗じた値の累計
		n = que.shift();   // 客の削除（待ち行列）
		base.lq_t = base.p_time;   // 現在の待ち行列長になった時刻
		let m  = base.Search(n);
		x1 = base.p_time - cus[m].time;
		base.c_wt += x1;   // 待ち時間の累計
		if (x1 > base.m_wt)
			base.m_wt = x1;   // 最大待ち時間
		let k = -1;
		for (let i1 = 0; i1 < base.s && k < 0; i1++) {
			if (base.sb[i1] == 0) {
				k = i1;
				base.sb[k]   = n;   // 窓口kの客番号
				base.st_e[k] = base.p_time + base.Next_sv();   // 窓口kのサービス終了時刻
				base.st_s[k] = base.p_time;   // 窓口kがふさがった時刻
				cus[m].state = k;   // 客の状態変更
			}
		}
	}
/*
          待ち行列がない場合
*/
	else {
		if (base.asb == 1)
			base.c_asb += (base.p_time - base.asb_t);   // すべての窓口がふさがっている時間の累計
	}
}

/****************************/
/* 指定した客の配列上の位置 */
/*      nm : 客番号         */
/*      return : 要素番号   */
/****************************/
Q_base.prototype.Search = function(nm)
{
	let n = -1;

	for (let i1 = 0; i1 < cus.length; i1++) {
		if (cus[i1].num == nm) {
			n = i1;
			break;
		}
	}

	return n;
}

/************************/
/* 結果の出力           */
/* （カッコ内は理論値） */
/************************/
Q_base.prototype.Output = function()
{
	let ram = 1.0 / base.m_a;
	let myu = 1.0 / base.m_s;
	let rou = ram / (base.s * myu);
	let pi, p0;
	if (base.s == 1) {
		p0 = 1.0 - rou;
		pi = rou;
	}
	else {
		p0 = 1.0 / (1.0 + 2.0 * rou + 4.0 * rou * rou / (2.0 * (1.0 - rou)));
		pi = 4.0 * rou * rou * p0 / (2.0 * (1.0 - rou));
	}
	let Lq = pi * rou / (1.0 - rou);
	let L  = Lq + base.s * rou;
	let Wq = Lq / ram;
	let W  = Wq + 1.0 / myu;
	let str = "シミュレーション時間＝" + Math.round(base.p_time) + " 客数＝" + base.nc + " ρ＝" + Math.round(1000*rou)/1000 + " ｐ0＝" + Math.round(1000*p0)/1000 + "\n";
	str += "   すべての窓口が塞がっている割合＝" + Math.round(1000*base.c_asb/base.p_time)/1000 + " (" + Math.round(1000*pi)/1000 + ")\n";
	str += "   各窓口が塞がっている割合\n";
	for (let i1 = 0; i1 < base.s; i1++)
		str += "      " + (i1+1) + "   " + Math.round(1000*base.c_sb[i1]/base.p_time)/1000 + "\n";
	str += "   平均待ち行列長＝" + Math.round(1000*base.c_lq_t/base.p_time)/1000 + " (" + Math.round(1000*Lq)/1000 + ")  最大待ち行列長＝" + base.m_lq + "\n";
	str += "   平均系内客数  ＝" + Math.round(1000*base.c_sc_t/base.p_time)/1000 + " (" + Math.round(1000*L)/1000 + ")  最大系内客数  ＝" + base.m_sc + "\n";
	str += "   平均待ち時間  ＝" + Math.round(1000*base.c_wt/base.nc)/1000 + " (" + Math.round(1000*Wq)/1000 + ")  最大待ち時間  ＝" + Math.round(base.m_wt) + "\n";
	str += "   平均滞在時間  ＝" + Math.round(1000*base.c_sys/base.nc)/1000 + " (" + Math.round(1000*W)/1000 + ")  最大滞在時間  ＝" + Math.round(base.m_sys) + "\n";

	document.getElementById("res").value = str;
}
